=begin
PureCloud Platform API

With the PureCloud Platform API, you can control all aspects of your PureCloud environment. With the APIs you can access the system configuration, manage conversations and more.

OpenAPI spec version: v2
Contact: DeveloperEvangelists@genesys.com
Generated by: https://github.com/swagger-api/swagger-codegen.git

License: UNLICENSED
https://help.mypurecloud.com/articles/terms-and-conditions/

Terms of Service: https://help.mypurecloud.com/articles/terms-and-conditions/

=end

require 'date'

module PureCloud
  # Details about the current state of a Flow execution
  class FlowRuntimeExecution
    # The flow execution ID
    attr_accessor :id

    # The flow execution name.
    attr_accessor :name

    # The Version of the flow definition of the flow execution.
    attr_accessor :flow_version

    # The time the flow was launched. Date time is represented as an ISO-8601 string. For example: yyyy-MM-ddTHH:mm:ss.SSSZ
    attr_accessor :date_launched

    # The flow's running status, which indicates whether the flow is running normally or completed, etc.
    attr_accessor :status

    # The time the flow completed, if applicable. Date time is represented as an ISO-8601 string. For example: yyyy-MM-ddTHH:mm:ss.SSSZ
    attr_accessor :date_completed

    # The completion reason set at the flow completion time, if applicable.
    attr_accessor :completion_reason

    # Additional information if the flow is in error
    attr_accessor :flow_error_info

    # List of the flow's output variables, if any. Output variables are only supplied for Completed flows.
    attr_accessor :output_data

    # The conversation to which this Flow execution is related
    attr_accessor :conversation

    # The URI for this object
    attr_accessor :self_uri

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'id' => :'id',
        
        :'name' => :'name',
        
        :'flow_version' => :'flowVersion',
        
        :'date_launched' => :'dateLaunched',
        
        :'status' => :'status',
        
        :'date_completed' => :'dateCompleted',
        
        :'completion_reason' => :'completionReason',
        
        :'flow_error_info' => :'flowErrorInfo',
        
        :'output_data' => :'outputData',
        
        :'conversation' => :'conversation',
        
        :'self_uri' => :'selfUri'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        
        :'id' => :'String',
        
        :'name' => :'String',
        
        :'flow_version' => :'FlowVersion',
        
        :'date_launched' => :'DateTime',
        
        :'status' => :'String',
        
        :'date_completed' => :'DateTime',
        
        :'completion_reason' => :'String',
        
        :'flow_error_info' => :'ErrorBody',
        
        :'output_data' => :'Hash<String, Object>',
        
        :'conversation' => :'DomainEntityRef',
        
        :'self_uri' => :'String'
        
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}){|(k,v), h| h[k.to_sym] = v}

      
      if attributes.has_key?(:'id')
        
        
        self.id = attributes[:'id']
        
      
      end

      
      if attributes.has_key?(:'name')
        
        
        self.name = attributes[:'name']
        
      
      end

      
      if attributes.has_key?(:'flowVersion')
        
        
        self.flow_version = attributes[:'flowVersion']
        
      
      end

      
      if attributes.has_key?(:'dateLaunched')
        
        
        self.date_launched = attributes[:'dateLaunched']
        
      
      end

      
      if attributes.has_key?(:'status')
        
        
        self.status = attributes[:'status']
        
      
      end

      
      if attributes.has_key?(:'dateCompleted')
        
        
        self.date_completed = attributes[:'dateCompleted']
        
      
      end

      
      if attributes.has_key?(:'completionReason')
        
        
        self.completion_reason = attributes[:'completionReason']
        
      
      end

      
      if attributes.has_key?(:'flowErrorInfo')
        
        
        self.flow_error_info = attributes[:'flowErrorInfo']
        
      
      end

      
      if attributes.has_key?(:'outputData')
        
        if (value = attributes[:'outputData']).is_a?(Array)
          self.output_data = value
        end
        
        
      
      end

      
      if attributes.has_key?(:'conversation')
        
        
        self.conversation = attributes[:'conversation']
        
      
      end

      
      if attributes.has_key?(:'selfUri')
        
        
        self.self_uri = attributes[:'selfUri']
        
      
      end

      
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properies with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      
      
      return invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      
      
      
      
      
      
      
      
      
      
      if @flow_version.nil?
        return false
      end

      
      
      
      
      
      if @date_launched.nil?
        return false
      end

      
      
      
      
      
      if @status.nil?
        return false
      end

      
      
      allowed_values = ["UNKNOWN", "RUNNING", "ERROR", "TERMINATED", "COMPLETED", "FAILED"]
      if @status && !allowed_values.include?(@status)
        return false
      end
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
    end

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] status Object to be assigned
    def status=(status)
      allowed_values = ["UNKNOWN", "RUNNING", "ERROR", "TERMINATED", "COMPLETED", "FAILED"]
      if status && !allowed_values.include?(status)
        fail ArgumentError, "invalid value for 'status', must be one of #{allowed_values}."
      end
      @status = status
    end

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          id == o.id &&
          name == o.name &&
          flow_version == o.flow_version &&
          date_launched == o.date_launched &&
          status == o.status &&
          date_completed == o.date_completed &&
          completion_reason == o.completion_reason &&
          flow_error_info == o.flow_error_info &&
          output_data == o.output_data &&
          conversation == o.conversation &&
          self_uri == o.self_uri
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, name, flow_version, date_launched, status, date_completed, completion_reason, flow_error_info, output_data, conversation, self_uri].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = Object.const_get("PureCloud").const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
