=begin
PureCloud Platform API

With the PureCloud Platform API, you can control all aspects of your PureCloud environment. With the APIs you can access the system configuration, manage conversations and more.

OpenAPI spec version: v2
Contact: DeveloperEvangelists@genesys.com
Generated by: https://github.com/swagger-api/swagger-codegen.git

License: UNLICENSED
https://help.mypurecloud.com/articles/terms-and-conditions/

Terms of Service: https://help.mypurecloud.com/articles/terms-and-conditions/

=end

require 'date'

module PureCloud
  class RecordingJobsQuery
    # Operation to perform bulk task
    attr_accessor :action

    # The date when the action will be performed. If the operation will cause the delete date of a recording to be older than the export date, the export date will be adjusted to the delete date. Date time is represented as an ISO-8601 string. For example: yyyy-MM-ddTHH:mm:ss.SSSZ
    attr_accessor :action_date

    # IntegrationId to Access AWS S3 bucket for bulk recording exports. This field is required and used only for EXPORT action.
    attr_accessor :integration_id

    # Include Screen recordings for export action, default value = true 
    attr_accessor :include_screen_recordings

    # Conversation Query. Note: After the recording is created, it might take up to 48 hours for the recording to be included in the submitted job query.
    attr_accessor :conversation_query

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'action' => :'action',
        
        :'action_date' => :'actionDate',
        
        :'integration_id' => :'integrationId',
        
        :'include_screen_recordings' => :'includeScreenRecordings',
        
        :'conversation_query' => :'conversationQuery'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        
        :'action' => :'String',
        
        :'action_date' => :'DateTime',
        
        :'integration_id' => :'String',
        
        :'include_screen_recordings' => :'BOOLEAN',
        
        :'conversation_query' => :'AsyncConversationQuery'
        
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}){|(k,v), h| h[k.to_sym] = v}

      
      if attributes.has_key?(:'action')
        
        
        self.action = attributes[:'action']
        
      
      end

      
      if attributes.has_key?(:'actionDate')
        
        
        self.action_date = attributes[:'actionDate']
        
      
      end

      
      if attributes.has_key?(:'integrationId')
        
        
        self.integration_id = attributes[:'integrationId']
        
      
      end

      
      if attributes.has_key?(:'includeScreenRecordings')
        
        
        self.include_screen_recordings = attributes[:'includeScreenRecordings']
        
      
      end

      
      if attributes.has_key?(:'conversationQuery')
        
        
        self.conversation_query = attributes[:'conversationQuery']
        
      
      end

      
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properies with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      
      
      return invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      
      
      if @action.nil?
        return false
      end

      
      
      allowed_values = ["DELETE", "EXPORT"]
      if @action && !allowed_values.include?(@action)
        return false
      end
      
      
      
      
      if @action_date.nil?
        return false
      end

      
      
      
      
      
      
      
      
      
      
      
      
      
      if @conversation_query.nil?
        return false
      end

      
      
      
      
    end

    
    
    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] action Object to be assigned
    def action=(action)
      allowed_values = ["DELETE", "EXPORT"]
      if action && !allowed_values.include?(action)
        fail ArgumentError, "invalid value for 'action', must be one of #{allowed_values}."
      end
      @action = action
    end

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          action == o.action &&
          action_date == o.action_date &&
          integration_id == o.integration_id &&
          include_screen_recordings == o.include_screen_recordings &&
          conversation_query == o.conversation_query
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [action, action_date, integration_id, include_screen_recordings, conversation_query].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = Object.const_get("PureCloud").const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
